import { get } from 'lodash'
import { withJournal } from 'xpub-journal'
import { SubmissionError } from 'redux-form'
import { create } from 'pubsweet-client/src/helpers/api'
import { loginUser } from 'pubsweet-component-login/actions'
import { compose, withState, withProps, withHandlers } from 'recompose'

import SignUpInvitation from './SignUpInvitationForm'

const login = (dispatch, values, history) =>
  dispatch(loginUser(values))
    .then(() => {
      history.push('/')
    })
    .catch(error => {
      const err = get(error, 'response')
      if (err) {
        const errorMessage = get(JSON.parse(err), 'error')
        throw new SubmissionError({
          password: errorMessage || 'Something went wrong',
        })
      }
    })

const confirmUser = (email, token, history) => (values, dispatch) => {
  const request = { ...values, email, token }
  if (values) {
    return create('/users/reset-password', request)
      .then(r => {
        const { username } = r
        const { password } = values
        login(dispatch, { username, password }, history)
      })
      .catch(error => {
        const err = get(error, 'response')
        if (err) {
          const errorMessage = get(JSON.parse(err), 'error')
          throw new SubmissionError({
            _error: errorMessage || 'Something went wrong',
          })
        }
      })
  }
}

export default compose(
  withJournal,
  withState('step', 'changeStep', 0),
  withProps(({ location }) => {
    const params = new URLSearchParams(location.search)
    const email = params.get('email')
    const token = params.get('token')
    const firstName = params.get('firstName')
    const lastName = params.get('lastName')
    const affiliation = params.get('affiliation')
    const title = params.get('title')

    return {
      initialValues: {
        email,
        firstName,
        lastName,
        affiliation,
        title,
        token,
      },
    }
  }),
  withHandlers({
    nextStep: ({ changeStep }) => () => changeStep(step => step + 1),
    prevStep: ({ changeStep }) => () => changeStep(step => step - 1),
    submitConfirmation: ({
      initialValues: { email, token },
      history,
      ...rest
    }) => confirmUser(email, token, history),
  }),
)(SignUpInvitation)
